import _defaults2 from 'lodash/defaults';
import _defaults from 'lodash/defaults';
import * as React from 'react';
import { connect } from 'react-redux';

import { loadFeature } from 'redux-features';

import { createSelector } from 'reselect';


export default function featureLoader(options) {

  const { getFeatures, getFeatureStates, featureId, render } = _defaults2({}, options, {
    getFeatures: state => state ? state.features : undefined,
    getFeatureStates: state => state ? state.featureStates : undefined
  });
  const mapStateToProps = createSelector(getFeatures, getFeatureStates, (features = {}, featureStates = {}) => ({
    feature: features[featureId],
    featureState: featureStates[featureId]
  }));

  function mergeProps({ feature, featureState }, { dispatch }, props) {
    return {
      dispatch,
      featureState,
      children: render ? render({ feature, featureState, props }) : null
    };
  }

  class FeatureLoader extends React.Component {
    componentWillMount() {
      const { featureState, dispatch } = this.props;
      if (featureState === 'NOT_LOADED') dispatch(loadFeature(featureId));
    }
    render() {
      return this.props.children;
    }
  }

  return connect(mapStateToProps, null, mergeProps)(FeatureLoader);
}